import { Player, world } from "mojang-minecraft";
import { config } from "../config.js";

type commandInfo = {
    /**
     * Name of the command
     */
    name: string,
    /**
     * Whether or not the command is admin only or not
     */
    admin?: boolean,
    /**
     * Aliases for the command
     */
    aliases?: string[]
    /**
     * Code to run when the command is ran
     */
    callback: (data: { player: Player, args: string[] }) => void
}

export const allCommands: Array<commandInfo> = [{
    name: 'test',
    admin: true,
    aliases: ["mycommand", "ad", "dauodsyha"],
    callback: (data) => {
        messagePlayer(data.player, `You executed the test command!`)
    }
}]


world.events.beforeChat.subscribe(data => {
    if (data.message.startsWith(config.commmandPrefix)) {
        data.cancel = true
        const args = data.message.slice(config.commmandPrefix.length).toLowerCase().split(/\s+/g)
        const command = args.shift()
        const commandData = allCommands.find(cmd => cmd.name.toLowerCase() === command || cmd.aliases?.includes(command))
        if (!commandData) return messagePlayer(data.sender, '§cInvalid command!')
        if (commandData.admin && !data.sender.hasTag(config.adminTag)) return messagePlayer(data.sender, `§cYou don't have permission to do that command!`)
        commandData.callback({ player: data.sender, args: args })
    }
})


/**
 * Send a message to a player
 * @param {Player} player Player to message
 * @param {string} message Message to send to the player
 */
const messagePlayer = (player: Player, message: string) => {
    player.runCommand(`tellraw @s {"rawtext":[{"text":"${message}"}]}`)
}