import { Entity, EntityInventoryComponent, Items, ItemStack, Player, world } from "mojang-minecraft";
import { ActionFormData } from "mojang-minecraft-ui";


world.events.beforeItemUse.subscribe(data => {
    if (data.source.id !== 'minecraft:player') return
    if (data.item.id === 'minecraft:recovery_compass' && data.item.nameTag === '§r§bShop') {
        shopGui(data.source as Player)
    }
})

const shopItems = [{
    name: 'Gold',
    cost: 50,
    item: new ItemStack(Items.get('minecraft:gold_ingot'))
}, {
    name: 'Diamond',
    cost: 100,
    item: new ItemStack(Items.get('minecraft:diamond'))
}]

const shopGui = (player: Player) => {
    const gui = new ActionFormData()
    gui.title(`Shop`)
    gui.body(`Choose what you want to buy!`)
    for (const item of shopItems) gui.button(`${item.name}: §2$${item.cost}`)
    gui.show(player).then(result => {
        if (result.isCanceled) return
        const money = getScore('Money', player, true)
        const item = shopItems[result.selection]
        if (money < item.cost) {
            messagePlayer(player, `§cYou don't have enough money to purchase that!`)
            return
        }
        messagePlayer(player, `§2You have purchased §a${item.name}§2 for §a$${item.cost}`);
        player.runCommand(`scoreboard players remove @s Money ${item.cost}`);
        (<EntityInventoryComponent>player.getComponent('inventory')).container.addItem(item.item);
    })
}


/**
 * Get the score of a target on an objective
 * @param {string} objective Objective to get a score from
 * @param {Entity|string} target The entity, player, or fake player to get the score of
 * @param {boolean} useZero Specifies whether to return NaN or 0 if an error is thrown
 * @returns {number} The target's score, or NaN / 0 if error
 */
const getScore = (objective: string, target: Entity | string, useZero: boolean = false): number => {
    try {
        const obj = world.scoreboard.getObjective(objective);
        if (typeof target == 'string') return obj.getScore(obj.getParticipants().find(v => v.displayName == target));
        return obj.getScore(target.scoreboard);
    } catch {
        return useZero ? 0 : NaN;
    }
}

/**
 * Send a message to a player
 * @param {Player} player Player to message
 * @param {string} message Message to send to the player
 */
const messagePlayer = (player: Player, message: string) => {
    player.runCommand(`tellraw @s {"rawtext":[{"text":"${message}"}]}`)
}